--[[

Booty Bay Gazette addon, v 6.2
https://www.bootybaygazette.com/

You should be able to query this DB from other addons:

    o={}
    TUJMarketInfo(2589,o)
    print(o['market'])

Prints the market price of Linen Cloth.
The item can be identified by anything GetItemInfo takes (itemid, itemstring, itemlink).

Prices are returned in copper.

    o['input']          -> the item parameter you just passed in, verbatim

    o['itemid']         -> the ID of the item you just passed in

    o['age']            -> number of seconds since data was compiled

    o['globalMedian']   -> median market price across all realms in this region
    o['globalMean']     -> mean market price across all realms in this region
    o['globalStdDev']   -> standard deviation of the market price across all realms in this region

    o['market']         -> average market price of the item on this AH over the past 14 days.
    o['stddev']         -> standard deviation of market price of the item on this AH over the past 14 days.
    o['recent']         -> average market price of the item on this AH over the past 3 days.

    o['days']           -> number of days since item was last seen on the auction house, when data was compiled. valid values 0 - 250.
     o['days'] = 251 means item was seen on this AH, but over 250 days ago
     o['days'] = 252 means the item is sold by vendors in unlimited quantities
     o['days'] = 255 means item was never seen on this AH


You can also query and set whether the additional tooltip lines appear from this addon.
This is useful for other addons (Auctioneer, TSM, etc) that have their own fancy tooltips to disable BBG tooltips and use BBG simply as a data source.

    TUJTooltip()        -> returns a boolean whether BBG tooltips are enabled
    TUJTooltip(true)    -> enables BBG tooltips
    TUJTooltip(false)   -> disables BBG tooltips

]]

local floor = math.floor
local tinsert, tonumber = tinsert, tonumber

local function coins(money)
    local GOLD="ffd100"
    local SILVER="e6e6e6"
    local COPPER="c8602c"

    local GSC_3 = "|cff%s%d|cff999999.|cff%s%02d|cff999999.|cff%s%02d|r"
    local GSC_2 = "|cff%s%d|cff999999.|cff%s%02d|r"

    money = floor(tonumber(money) or 0)
    local g = floor(money / 10000)
    local s = floor(money % 10000 / 100)
    local c = money % 100

    if (g > 0) then
        return GSC_3:format(GOLD, g, SILVER, s, COPPER, c)
    else
        return GSC_2:format(SILVER, s, COPPER, c)
    end
end

local function char2dec(s)
    local n, l = 0, string.len(s)
    for i=1,l,1 do
        n = n * 256 + string.byte(s,i)
    end
    return n
end

local function round(num)
    return floor(num + 0.5)
end

local addonName, addonTable = ...

local marketInfoCache, marketInfoCacheKeys, marketInfoCacheMaxDepth = {}, {}, 16

local function InfoCacheKeyComparitor(x,y)
    return marketInfoCache[x]['__queried'] < marketInfoCache[y]['__queried']
end

--[[
    pass a table as the second argument to wipe and reuse that table
    otherwise a new table will be created and returned
]]
function TUJMarketInfo(item,...)
    if item == 0 then
        if addonTable.marketData then
            return true
        else
            return false
        end
    end

    local numargs = select('#', ...)
    local tr

    if (numargs > 0) and (type(select(1,...)) == 'table') then
        tr = select(1,...)
        wipe(tr)
    end

    if not addonTable.marketData then return tr end

    if marketInfoCache[item] then
        marketInfoCache[item]['__queried'] = GetTime()
        if not marketInfoCache[item]['input'] then return tr end
        if not tr then tr = {} end
        for k,v in pairs(marketInfoCache[item]) do
            tr[k] = v
        end
        tr['__queried'] = nil
        return tr
    end

    if #marketInfoCacheKeys >= marketInfoCacheMaxDepth then
        -- reuse oldest cache table
        sort(marketInfoCacheKeys, InfoCacheKeyComparitor)
        marketInfoCache[item] = marketInfoCache[marketInfoCacheKeys[1]]
        wipe(marketInfoCache[item])

        while #marketInfoCacheKeys >= marketInfoCacheMaxDepth do
            marketInfoCache[marketInfoCacheKeys[1]] = nil
            table.remove(marketInfoCacheKeys, 1)
        end
    else
        -- add new table to cache
        marketInfoCache[item] = {}
    end
    table.insert(marketInfoCacheKeys, item)
    marketInfoCache[item]['__queried'] = GetTime()

    local _, link, dataKey
    local iid, pricingLevel, species, quality

    iid = GetItemInfoInstant(item)
    if not iid then return tr end

    dataKey = tostring(iid)
    local dta = addonTable.marketData[dataKey]
    if not dta then return tr end

    if not tr then tr = {} end

    tr['input'] = item
    tr['itemid'] = iid

    if addonTable.dataAge then
        tr['age'] = time() - addonTable.dataAge
    end

    local priceSize = string.byte(dta, 1)

    local offset = 2

    tr['globalMedian'] = round(char2dec(string.sub(dta, offset, offset+priceSize-1)))
    if tr['globalMedian'] == 0 then tr['globalMedian'] = nil end
    offset = offset + priceSize

    tr['globalMean'] = round(char2dec(string.sub(dta, offset, offset+priceSize-1)))
    offset = offset + priceSize

    tr['globalStdDev'] = round(char2dec(string.sub(dta, offset, offset+priceSize-1)))
    offset = offset + priceSize

    tr['days'] = string.byte(dta, offset, offset)
    offset = offset + 1

    tr['market'] = round(char2dec(string.sub(dta, offset, offset+priceSize-1)))
    offset = offset + priceSize

    tr['stddev'] = round(char2dec(string.sub(dta, offset, offset+priceSize-1)))
    offset = offset + priceSize

    tr['recent'] = round(char2dec(string.sub(dta, offset, offset+priceSize-1)))
    --offset = offset + priceSize

    for k,v in pairs(tr) do
        marketInfoCache[item][k] = v
    end

    return tr
end

--[[
    enable/disable/query whether the BBG tooltip additions are enabled
]]
local tooltipsEnabled = true
function TUJTooltip(...)
    if select('#', ...) >= 1 then
        tooltipsEnabled = not not select(1,...) --coerce into boolean
    end
    return tooltipsEnabled
end

local LibExtraTip = LibStub("LibExtraTip-1")

local function buildExtraTip(tooltip, pricingData)
    local r,g,b = .9,.8,.5

    LibExtraTip:AddLine(tooltip," ",r,g,b,true)

    if (pricingData['age'] > 3*24*60*60) then
        LibExtraTip:AddLine(tooltip,"As of "..SecondsToTime(pricingData['age'],pricingData['age']>60).." ago:",r,g,b,true)
    end

    if pricingData['recent'] then
        LibExtraTip:AddDoubleLine(tooltip,"3-Day Price",coins(pricingData['recent']),r,g,b,nil,nil,nil,true)
    end
    if pricingData['market'] then
        LibExtraTip:AddDoubleLine(tooltip,"14-Day Price",coins(pricingData['market']),r,g,b,nil,nil,nil,true)
    end
    if pricingData['stddev'] then
        LibExtraTip:AddDoubleLine(tooltip,"14-Day Std Dev",coins(pricingData['stddev']),r,g,b,nil,nil,nil,true)
    end

    local regionName = addonTable.region or "Regional"

    if pricingData['globalMedian'] then
        LibExtraTip:AddDoubleLine(tooltip,regionName.." Median",coins(pricingData['globalMedian']),r,g,b,nil,nil,nil,true)
    end
    if pricingData['globalMean'] then
        LibExtraTip:AddDoubleLine(tooltip,regionName.." Mean",coins(pricingData['globalMean']),r,g,b,nil,nil,nil,true)
    end
    if pricingData['globalStdDev'] then
        LibExtraTip:AddDoubleLine(tooltip,regionName.." Std Dev",coins(pricingData['globalStdDev']),r,g,b,nil,nil,nil,true)
    end

    if pricingData['days'] == 255 then
        LibExtraTip:AddLine(tooltip,"Never seen here before",r,g,b,true)
    elseif pricingData['days'] == 252 then
        LibExtraTip:AddLine(tooltip,"Sold by Vendors",r,g,b,true)
    elseif pricingData['days'] > 250 then
        LibExtraTip:AddLine(tooltip,"Last seen over 250 days ago",r,g,b,true)
    elseif pricingData['days'] > 1 then
        LibExtraTip:AddLine(tooltip,"Last seen "..SecondsToTime(pricingData['days']*24*60*60).." ago",r,g,b,true)
    end
end

local dataResults = {}

local function onTooltipSetItem(tooltip, itemLink, quantity)
    if not addonTable.marketData then return end
    if not tooltipsEnabled then return end
    if not itemLink then return end

    TUJMarketInfo(itemLink, dataResults)

    if not (dataResults['input'] and (dataResults['input'] == itemLink)) then
        return
    end

    buildExtraTip(tooltip, dataResults)
end

local eventframe = CreateFrame("FRAME",addonName.."Events")

local function onEvent(self,event,arg)
    if event == "PLAYER_ENTERING_WORLD" then
        eventframe:UnregisterEvent("PLAYER_ENTERING_WORLD")

        local realmId
        local guid = UnitGUID("player")
        local faction = UnitFactionGroup("player")
        if guid and faction then
            realmId = strmatch(guid, "^Player%-(%d+)") .. string.sub(faction, 1, 1)
        end
        if not realmId then
            realmId = "nil"
        else
            for i=1,#addonTable.dataLoads,1 do
                addonTable.dataLoads[i](realmId)
                addonTable.dataLoads[i]=nil
            end
        end
        wipe(addonTable.dataLoads)
        collectgarbage("collect") -- lots of strings made and trunc'ed in MarketData

        if not addonTable.realmIndex then
            print("Booty Bay Gazette - Warning: could not find data for realm ID "..realmId..", no data loaded!")
        elseif not addonTable.marketData then
            print("Booty Bay Gazette - Warning: no data loaded!")
        else
            if not tooltipsEnabled then
                print("Booty Bay Gazette - Tooltip prices disabled. Run /bbgtooltip to toggle.")
            end
            LibExtraTip:AddCallback({type = "item", callback = onTooltipSetItem, allevents = true})
            LibExtraTip:RegisterTooltip(GameTooltip)
            LibExtraTip:RegisterTooltip(ItemRefTooltip)
        end
    elseif event == "ADDON_LOADED" then
        tooltipsEnabled = not _G["BBGTooltipsHidden"]
    elseif event == "PLAYER_LOGOUT" then
        _G["BBGTooltipsHidden"] = not tooltipsEnabled
    end
end

eventframe:RegisterEvent("PLAYER_ENTERING_WORLD")
eventframe:RegisterEvent("ADDON_LOADED")
eventframe:RegisterEvent("PLAYER_LOGOUT")
eventframe:SetScript("OnEvent", onEvent)

SLASH_BOOTYBAYGAZETTE1 = '/bbgtooltip'
function SlashCmdList.BOOTYBAYGAZETTE(msg)
    local newEnabled = not TUJTooltip()
    if msg == 'on' then
        newEnabled = true
    elseif msg == 'off' then
        newEnabled = false
    end
    if TUJTooltip(newEnabled) then
        print("Booty Bay Gazette - Tooltip prices enabled.")
    else
        print("Booty Bay Gazette - Tooltip prices disabled.")
    end
end

local origGetAuctionBuyout = GetAuctionBuyout
local getAuctionBuyoutTable = {}

function GetAuctionBuyout(item) -- Tekkub's API
    local result = TUJMarketInfo(item, getAuctionBuyoutTable)
    if (result and result['input'] == item) then
        if (result['market']) then
            return result['market']
        else
            return result['globalMedian']
        end
    end

    if (origGetAuctionBuyout) then
        return origGetAuctionBuyout(item)
    end

    return nil
end


